/********************************************************************* 
 * Java Based e-AR Lite interface for Activity sensing (ie 4 Hz)
 * (using Java usblib wrapper) http://libusbjava.sourceforge.net/wp/?page_id=8
 *
 * by Benny Lo
 * Mar 7 2012
 *
 *********************************************************************/
import ch.ntb.usb.Device;
import ch.ntb.usb.USB;
import ch.ntb.usb.USBException;
import ch.ntb.usb.LibusbJava;
import ch.ntb.usb.Usb_Bus;
import ch.ntb.usb.Usb_Device_Descriptor;
import ch.ntb.usb.Usb_Device;
import java.io.FileWriter;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.*;

public class eARLiteInterface {

  public static final int MAX_USB_BUSES=30;                         //maximum no. of usb buses 
  public static final int MAX_NO_USB_DEVICES = 100;                 //maximum no of usb devices
  public static final int USB_BUFFER_SIZE=60;                       //buffer size for usb transfer
  public static final int EAR_LITE_VENDOR_ID=0x1915;                //vendor ID of the e-AR lite sensor
  public static final int EAR_LITE_PRODUCT_ID=0x1102;               //product ID of the e-AR lite sensor
  public static final byte FLASH_COMMAND_STATUS			='a';		//check connection
  public static final byte FLASH_COMMAND_START			='b';		//initiate the download
  public static final byte FLASH_COMMAND_NEXT_BLOCK		='d';		//download the next block of data
  public static final byte FLASH_COMMAND_ERASE_ALL		='E';		//erase all memory
  public static final byte FLASH_COMMAND_ERASE_SECTOR	='e';		//erase a sector
  public static final byte FLASH_COMMAND_INIT			='s';		//initialised the SPI interface with Flash memory
  public static final byte FLASH_COMMAND_STOP			='S';		//stop the SPI interface with flash memory
  public static final byte FLASH_COMMAND_READ_FROM_MSP	='c';		//retrieve data from MSP
  public static final byte FLASH_COMMAND_TO_MSP			='k';		//send a command to MSP
  public static final byte FLASH_COMMAND_QUIT			='q';		//disconnect the sensor from USB
  public static final byte FLASH_COMMAND_FIND_AVAILABLE	='f';		//Find the first available free memory address
  public static final byte MSP_INTERFACE_CHECK_CONNECTION  =0x10;	//check the SPI connection to MSP
  public static final byte MSP_INTERFACE_READ_CLOCK        =0x11;    //read the real time clock
  public static final byte MSP_INTERFACE_SET_CLOCK         =0x12;    //set the real time clock
  public static final byte MSP_INTERFACE_ERASE_MEMORY      =0x13;    //erase all memory
  public static final byte MSP_INTERFACE_EEPROM_ACCESS     =0x14;    //Nordic request EEPROM access -> (ie power up the nRF24)
  public static final byte MSP_INTERFACE_EEPROM_RELEASE    =0x15;    //Nordic release EEPROM access -> (ie power down the nRF24)
  public static final byte MSP_INTERFACE_READ_CLOCK_REQUEST=0x16;	//request to read the real time cloc
  public static final int FLASH_MAX_MEMORY_SIZE=16777216;
  public static final int FLASH_SECTION_SIZE=65536;
  public static final int FLASH_PAGE_SIZE=256;
  public static final int FLASH_NO_SECTIONS=FLASH_MAX_MEMORY_SIZE/FLASH_SECTION_SIZE;
//EEPROM Access functions
  public static final byte EEPROM_ACCESS_DOWNLOAD_DATA=1;
  public static final byte EEPROM_ACCESS_ERASE_SECTIONS=2;
  public static final byte EEPROM_ACCESS_ERASE_ALL_MEMMORY=3;
  public static final byte EEPROM_ACCESS_GET_FIRST_AVAILABLE_ADDR=4;
  
  private static byte[] senddatabuf;
  private static boolean usbinterface_ready;
  public static int flash_status;//0=checking connection,1=start download,2=downloading page
  private static int currentMSPcommand;
  private static boolean flash_init,msp_connection_ready,read_data_from_MSP;
  private static long usb_device_handle;
  private static boolean foundfirstblock=false;
  private static FileOutputStream fout = null;
  private static boolean EEPROMaccess=false;
  private static int request=0;
  private static boolean downloading=false;
  private static int downloadingsize=0;
  private static int lasteofsectionsize=0;
  private static int nosections=0;
  private static int cursections=0;
  private static boolean ready_to_convertdata=false;
  private static boolean erasing=false;
  private static int curerasesection=0;
  private static String eraseaddr="0";
  private static int preparing=0;
  public static void println(String str)
  {
	  System.out.println(str);
  }
  public static void print(String str)
  {
	  System.out.print(str);
  }
  public static boolean SendFlashCommand(byte command)
  { 
	 if (!usbinterface_ready) return false;
	 senddatabuf[0]=command;
	 senddatabuf[1]=0;
	 flash_status=command;		 		 
	 //println("Sent command:"+(char)command);
	// int byteswritten=LibusbJava.usb_interrupt_write(usb_device_handle,1,senddatabuf,USB_BUFFER_SIZE,5000);
	 int byteswritten=LibusbJava.usb_bulk_write	(usb_device_handle,1,senddatabuf,USB_BUFFER_SIZE,5000);
	 if (byteswritten <0)
	 {
		 println("Error in sending data via USB");
		 return false;
	 }
	 return true;
  }
  public static boolean SendFlashCommand(byte command,byte[] data,int size)
  {
	  int i;
	  if (!usbinterface_ready) return false;	
	  senddatabuf[0]=command;
	  for (i=0;i<Math.min(USB_BUFFER_SIZE,size);i++)
		  senddatabuf[i+1]=data[i];
	  for (i=size;i<USB_BUFFER_SIZE-1;i++)
		  senddatabuf[i+1]=0;
	  flash_status=command;	
	  //println("Sent command:"+(char)command+" no bytes:"+String.format("%d",size));
	  //int byteswritten=LibusbJava.usb_interrupt_write(usb_device_handle,1,senddatabuf,USB_BUFFER_SIZE,5000);
	  int byteswritten=LibusbJava.usb_bulk_write(usb_device_handle,1,senddatabuf,USB_BUFFER_SIZE,5000);
	  if (byteswritten <0)
	  {
		  println("Error in sending data via USB");
		  return false;
	  }
		return true;
  }
  public static boolean SendMSPCommand(byte mspcommand)
  {
	  byte[] mspdata=new byte[2];
	  currentMSPcommand=mspcommand;
	  mspdata[0]=mspcommand;
	  mspdata[1]=0;
	  //println("Sent MSP command:0x"+Integer.toHexString(mspcommand&0xff));
	  return SendFlashCommand(FLASH_COMMAND_READ_FROM_MSP,mspdata,2);
  }
  public static String GetCurrentTimeString()
  {
	  	Calendar cal = Calendar.getInstance();
	  	//cal.setTimeZone(TimeZone.getTimeZone("GMT"));	   	
	    return String.format("%04d%02d%02d %02d%02d%02d",cal.get(Calendar.YEAR),cal.get(Calendar.MONTH)+1,cal.get(Calendar.DATE),
	    		cal.get(Calendar.HOUR_OF_DAY),cal.get(Calendar.MINUTE),cal.get(Calendar.SECOND));
  }
  public static boolean SetRealTimeClock()
  {//set the real-time clock
	  	Calendar cal = Calendar.getInstance();
	  	cal.setTimeZone(TimeZone.getTimeZone("GMT"));	   
	    int year=cal.get(Calendar.YEAR);
	    int month=cal.get(Calendar.MONTH)+1;
	    int date=cal.get(Calendar.DATE);
	    int hour=cal.get(Calendar.HOUR_OF_DAY);
	    int minute=cal.get(Calendar.MINUTE);
	    int second=cal.get(Calendar.SECOND);
	    println(String.format("%02d/%02d/%02d %02d:%02d:%02d",date,month,year%100,hour,minute,second));
	    currentMSPcommand=MSP_INTERFACE_SET_CLOCK;
	    flash_status=FLASH_COMMAND_TO_MSP;
	    senddatabuf[0]=FLASH_COMMAND_TO_MSP;
		senddatabuf[1]=MSP_INTERFACE_SET_CLOCK;
		senddatabuf[2]=(byte)date;
		senddatabuf[3]=(byte)month;
		senddatabuf[4]=(byte)(year-2000);
		senddatabuf[5]=(byte)(hour+1);
		senddatabuf[6]=(byte)(minute+1);
		senddatabuf[7]=(byte)(second+1);
		senddatabuf[8]=0;
//		int byteswritten=LibusbJava.usb_interrupt_write(usb_device_handle,1,senddatabuf,USB_BUFFER_SIZE,5000);
		int byteswritten=LibusbJava.usb_bulk_write(usb_device_handle,1,senddatabuf,USB_BUFFER_SIZE,5000);
		  if (byteswritten <0)
		  {
			  println("Error in sending data via USB");
			  return false;
		  }
	  return true;
  }
  private static void helpmenu()
  {
	  println("----e-AR Lite Activity Sensing Inteface Commands------");
	  println("q - quit");
	  println("r - read RTC clock from sensor");		
	  println("s - sync sensor clock with PC");
	  println("d - download data from sensor");
	  println("e - erase all memory");
	  //println("f - find first available memory address");
	  println("p - prepare the sensor : erase memory and synch sensor clock");
	  println("h - help");
  }
  private static void createFile(String filename)
  {
	  try{
		  // Create file 
		  fout = new FileOutputStream(filename);		  
	  }catch (Exception e){//Catch exception if any
		  System.err.println("Error: " + e.getMessage());
	  }
  }
  private static void RequestEEPROMAccess(int currequest)
  {	//request the access to the Flash memory
  	if (!EEPROMaccess)
  	{
  		request=currequest;
  		println("Sent EEPROM access request");
  		SendMSPCommand(MSP_INTERFACE_EEPROM_ACCESS);  		
  	}
  }
  private static void ConvertBinaryToCSVFile(String sourcefile,String destinatedfile)
  {//convert the file to CSV file
	  FileInputStream fin = null;
	  FileWriter ffout=null;
	  BufferedWriter bfout=null;
	  String curtimestr=GetCurrentTimeString();
	  String newfilename=destinatedfile;
	  int endofstr=destinatedfile.indexOf('.');
	  newfilename=destinatedfile.substring(0,endofstr) +" "+ curtimestr+".csv";		  
	  try {
		  fin = new FileInputStream(sourcefile);
		  int filesize=fin.available()-lasteofsectionsize;
		  if (filesize <10) 
		  {	//nothing to write
			  fin.close();
			  println("No sensor data is found!");
			  return;
		  }
		  println("Write to file \'"+newfilename+"\'");
		  ffout=new FileWriter(newfilename);
		  bfout= new BufferedWriter(ffout);		  
		  byte []buffer=new byte[filesize];
		  fin.read(buffer,0,filesize);
		  bfout.write("e-AR Lite Activity (4Hz),v1\n");
  		  bfout.write("Date,Time,Counter,X,Y,Z\n");
  		  int timestampcount=0;
  		  boolean istimestamp=false;
  		  byte previousbyte=0;
  		  int day,month,year,hour,minute,second;
  		  day=1;month=1;year=10;hour=0;minute=0;second=0;
  		  int x=0,y=0,z=0,a=0;
  		  int bytecount=0;
  		  int valuecount=0;
  		  String timestampstr="0/0/0,0:0:0";
  		  int nolines=0;			
  		  int pagesize=(int)(FLASH_PAGE_SIZE/3)*3;
  		  int subsecond=0;
  		  for (int i=0;i<filesize;i++)
  		  {
  			if (((i%FLASH_PAGE_SIZE)==0) && i!=0)//each page
  			{
  				valuecount=0;
  				bytecount=0;
  			}
  			if (bytecount<pagesize)
  			{
  				if (!istimestamp)
  				{
  					if ((buffer[i]&0xff)==0)
  						timestampcount++;
  					else if ((buffer[i]&0xff)!=0)
  						timestampcount=0;
  					if (timestampcount >=3 && !istimestamp)//it's a timestamp
  					{
  						timestampcount=0;
  						istimestamp=true;
  					}
  				}
  				if (istimestamp)
  				{
  					switch (timestampcount)
  					{
  						case 0:timestampcount++;break;
  						case 1:	day=(buffer[i]&0xff);timestampcount++;break;
  						case 2: month=(buffer[i]&0xff);timestampcount++;break;
  						case 3: year=(buffer[i]&0xff);timestampcount++;break;
  						case 4: hour=(buffer[i]&0xff);timestampcount++;break;
  						case 5: minute=(buffer[i]&0xff);timestampcount++;break;
  						case 6: second=(buffer[i]&0xff);timestampcount=0;istimestamp=false;
  							if (year >=10 && day <=31 && month<=12 && month>0 && hour<=24 && hour>=0 && minute<=60 && minute>0 && second>=0 && second<=60)
  							{
  								timestampstr= String.format("%d/%d/%d,%d:%d:%d",day,month,year,hour,minute,second);
  								subsecond=0;
  							}break;
  					}				
  					valuecount=0;
  				}
  				else {				
  					valuecount++;  					
  					if (valuecount==1)
  						x=(buffer[i]&0xff);							
  					else if (valuecount==2)
  						y=(buffer[i]&0xff);
  					else if (valuecount==3)
  					{
  						z=(buffer[i]&0xff);					
  						valuecount=0;	
  						String str=String.format("%02d/%02d/%02d,%02d:%02d:%02d,%d,%d,%d,%d\n",
							day,month,year,hour,minute,second,subsecond,x,y,z);
  						bfout.write(str);
  						subsecond++;//counter for indicate samples per 3 seconds
  						nolines++;  							  						
  					}					
  				}
  			}
  			bytecount++;
  			previousbyte=buffer[i];
  		  }  		
  		  bfout.close();
  		  ffout.close();
  		  fin.close();
	  }
	  catch (Exception e)
	  {
		  System.err.println("Error:"+e.getMessage());
	  }  	  		
  }
  private static void Parse(byte[] data,int len) 
  {//Parse data received from the e-AR Lite sensor via the USB
	  int endofstr;String str;
	  int i=0;
	  int eof=0;//end of file    
	switch (flash_status)
	{
		case FLASH_COMMAND_STATUS:	str = new String(data);
									endofstr=str.indexOf(0);
									println("e-AR Lite Sensor - "+str.substring(0,endofstr));
									SendFlashCommand(FLASH_COMMAND_INIT);
									break;
		case FLASH_COMMAND_INIT: 	if (data[0]==FLASH_COMMAND_INIT)
									{
										println("Initialised!");
										flash_init=true;
									}break;
		case FLASH_COMMAND_READ_FROM_MSP:
									//print("received:");
									//str = new String(data);
									//endofstr=str.indexOf(0);
									//println(str.substring(0,endofstr));
									if (data[0] == 'm' && currentMSPcommand==MSP_INTERFACE_CHECK_CONNECTION)
									{//KillTimer(TIMER_MSP_CONNECTION);
										println("e-AR Lite interface is ready");
										msp_connection_ready=true;//the e-AR lite sensor is ready now!
										helpmenu();
									}
									else if (data[0]=='r' && currentMSPcommand==MSP_INTERFACE_READ_CLOCK_REQUEST)
									{
										println("Read Real-time clock");				
										SendMSPCommand(MSP_INTERFACE_READ_CLOCK);
									}
									else if (currentMSPcommand==MSP_INTERFACE_READ_CLOCK)
									{
										if (data[0]=='r' && data[1]=='e' && data[2]=='a')
										{
											SendMSPCommand(MSP_INTERFACE_READ_CLOCK);
										}
										else {
											read_data_from_MSP=false;										
											String msg=String.format("%02d/%02d/%02d %02d:%02d:%02d",data[0],data[1],data[2],data[3]-1,data[4]-1,data[5]-1);
											println(msg);
										}
									}
									else if (currentMSPcommand==MSP_INTERFACE_EEPROM_ACCESS)
									{
											println("Flash memory access granted");
											EEPROMaccess=true;
											switch (request)
											{
												case EEPROM_ACCESS_DOWNLOAD_DATA:SendFlashCommand(FLASH_COMMAND_START);break;
												case EEPROM_ACCESS_ERASE_SECTIONS:case EEPROM_ACCESS_ERASE_ALL_MEMMORY:
													eraseaddr=String.format("0");
													byte[] bytearray = eraseaddr.getBytes();
													curerasesection=1;
													SendFlashCommand(FLASH_COMMAND_ERASE_SECTOR,bytearray,bytearray.length);
													break;
												case EEPROM_ACCESS_GET_FIRST_AVAILABLE_ADDR:SendFlashCommand(FLASH_COMMAND_FIND_AVAILABLE);break;
											}
											
									}
									else if (currentMSPcommand==MSP_INTERFACE_EEPROM_RELEASE)
									{
										if (data[0] == 'r' && EEPROMaccess)//released
										{
											println("EEPROM Released");
											EEPROMaccess=false;			
											read_data_from_MSP=false;
											if (request == EEPROM_ACCESS_DOWNLOAD_DATA)
												ready_to_convertdata=true;
											request=0;
										}
										else SendMSPCommand(MSP_INTERFACE_EEPROM_RELEASE);	
									}
									break;
		case FLASH_COMMAND_TO_MSP:	//sent a command to MSP
									if (currentMSPcommand==MSP_INTERFACE_SET_CLOCK)
									{
										read_data_from_MSP=false;
										println("Clock set");										
									}
									break;
		case FLASH_COMMAND_START:	println("Memory download started");
									foundfirstblock=false;
									SendFlashCommand(FLASH_COMMAND_NEXT_BLOCK);											
									break;//started download
		case FLASH_COMMAND_NEXT_BLOCK://downloading page							
			if (fout !=null && downloading)
			{				
				downloadingsize+=len;				
				for (i=0;i<len && !foundfirstblock;i++)
				{
						if ((data[i]&0xff)!=0xff)
						{
							foundfirstblock=true;
							eof=0;							
						}
				}				
				if (foundfirstblock)
				{
					try {
						fout.write(data,0,len);
					}
					catch (Exception e)
					{
						 System.err.println("Error: " + e.getMessage());
						 break;
					}
					for (i=0;i<len;i++)
					{
						if ((data[i]&0xff) == 0xff)//end of file detected
							eof++;							
						if (eof>0 && (data[i]&0xff)!=0xff)
							eof=0;
						if (eof>16)
						{
							lasteofsectionsize=(len-i+16);//remove the end of file indicators
							break;
						}
					}
				}							
				nosections=(downloadingsize/65536)+1;
			}
			if (downloadingsize< FLASH_MAX_MEMORY_SIZE && eof<16)
			{
				if (cursections != nosections)
				{
					print(".");
					cursections=nosections;					
				}
				SendFlashCommand(FLASH_COMMAND_NEXT_BLOCK);
			}
			else {				
				println("Finished download");
				try{
				 if (fout!=null)
			    	  fout.close();
				 fout=null;
			     }
			     catch (Exception e){//Catch exception if any
					  System.err.println("Error: " + e.getMessage());
				  }
				SendFlashCommand(FLASH_COMMAND_STOP);
			}
			break;
		case FLASH_COMMAND_ERASE_SECTOR:
						//str = new String(data);
						//endofstr=str.indexOf(0);
						//println(str.substring(0,endofstr));
					if (data[0]=='e')						
					{
						if (curerasesection>=nosections)						
						{
							println("\nMemory sector erased!");							
							erasing=false;
							SendMSPCommand(MSP_INTERFACE_EEPROM_RELEASE);	
						}
						else {							
							print(".");
							String addr=String.format("%d",curerasesection);
							byte[] bytearray = addr.getBytes();							
							SendFlashCommand(FLASH_COMMAND_ERASE_SECTOR,bytearray,bytearray.length);
							curerasesection++;
						}
					}
					break;
		case FLASH_COMMAND_ERASE_ALL: 	if (data[0] == 'e' && data[1]=='r' && data[2]=='a')//erased
										{
											println("Memory erase completed");
											erasing=false;
											if (EEPROMaccess)
												SendMSPCommand(MSP_INTERFACE_EEPROM_RELEASE);	
										}
										break;
		case FLASH_COMMAND_STOP:	 if (data[0]==FLASH_COMMAND_STOP)
		 							{
										println("Flash Memory stopped!");
										SendMSPCommand(MSP_INTERFACE_EEPROM_RELEASE);
		 							}
									break;
		case FLASH_COMMAND_QUIT:	if (data[0] == FLASH_COMMAND_QUIT)
										println("USB Connection disabled!");
									break;
		case FLASH_COMMAND_FIND_AVAILABLE:
									if (data[0] == FLASH_COMMAND_FIND_AVAILABLE)
									{
										print("First available address: ");
										long longaddr=data[1]*16777216+data[2]*65536+data[3]*256+data[4];
										str=String.format("%d",longaddr);
										//str=String.format("%d,%d,%d,%d,%d",data[1],data[2],data[3],data[4],data[5]);
										println(str);
										SendMSPCommand(MSP_INTERFACE_EEPROM_RELEASE);	
									}
									break;
								
	}
  }

  public static void main(String[] args) {
	int i;
	byte option;
	senddatabuf = new byte[USB_BUFFER_SIZE];
	option=0;
	read_data_from_MSP=false;
	flash_init=false;
	foundfirstblock=false;
	flash_status=0;currentMSPcommand=0;msp_connection_ready=false;
	usbinterface_ready=false;usb_device_handle=0;
	println("-----------------------------------");
	println("Sensixa e-AR Lite Interface Program");
	println("-----------------------------------");
	LibusbJava.usb_init();
    LibusbJava.usb_find_busses();
    LibusbJava.usb_find_devices();
    Usb_Bus []buslist=new Usb_Bus[MAX_USB_BUSES];
    Usb_Bus bus = LibusbJava.usb_get_busses();
    int nobuses=0;
    Usb_Device[] devicelist=new Usb_Device[MAX_NO_USB_DEVICES];
    int nodevices=0;
    while (bus!=null)
    {
        buslist[nobuses++]=bus;
        //println("bus:"+bus.getDirname());
        Usb_Device pdevice;
        pdevice=bus.getDevices();
        while (pdevice !=null)
        {   
            devicelist[nodevices]=pdevice;    	
    	
            Usb_Device_Descriptor describ=pdevice.getDescriptor();
            int vendorID=describ.getIdVendor();
            int product=describ.getIdProduct();
          //  println("Vendor: 0x"+Integer.toHexString(vendorID)+" Product: 0x"+Integer.toHexString(product&0xffff));
            if (vendorID == EAR_LITE_VENDOR_ID && product== EAR_LITE_PRODUCT_ID)
            {
                println("Device ["+nodevices+"]:"+pdevice.getFilename());
                int manufacturerID=describ.getIManufacturer();
                int productID=describ.getIProduct();
                long pdev=LibusbJava.usb_open(pdevice);
		//println("Manufacturer ID:"+manufacturerID+" product id:"+productID);
                print(LibusbJava.usb_get_string_simple(pdev,manufacturerID));
                println(" "+LibusbJava.usb_get_string_simple(pdev,productID));
                LibusbJava.usb_close(pdev);
                //System.out.print("Serial No:"+Integer.toHexString(describ.getISerialNumber())+"\n");
                nodevices++;
            }
            pdevice=pdevice.getNext();				
        } 
        bus=bus.getNext();
    }
    if (nodevices <=0)
    {
    	println("No e-AR Lite sensor is found!");
    	return;
    }
    int whichdevice=0;
    if (nodevices>1)
    {
    	System.out.print("Which device to open? (0-"+(nodevices-1)+"):");
    	try {
    		InputStreamReader isr = new InputStreamReader(System.in);
    		BufferedReader br = new BufferedReader(isr);
    		String s = br.readLine();
    		 whichdevice =  Integer.valueOf(s);
    	}
    	catch (IOException e) 
    	{ whichdevice=0;   	  
    	}
    	
    }
    
    
   // get a device instance with vendor id and product id
  // Device dev = USB.getDevice((short) 0x1915, (short) 0x1102);//e-AR lite device
    //try {         	
      // data read from the device
      byte[] readData = new byte[USB_BUFFER_SIZE];
      for (i=0;i<USB_BUFFER_SIZE;i++)
    	  readData[i]=0;
	
      // open the device with configuration 1, interface 0 and without altinterface
    //dev.open(1, 0, -1);
	  usb_device_handle=LibusbJava.usb_open(devicelist[whichdevice]);
	  if (usb_device_handle ==0)
	  {
		  println("Error in openning the USB device");
		  return;
	  }	  
	  	// write some data to the device
    // dev.writeInterrupt(1, data, 64, 5000, false);
	  if (LibusbJava.usb_claim_interface(usb_device_handle,0)<0)
	  {
		  println("Error in claiming interface!");
		  return;
	  }
	  usbinterface_ready=true;
	 SendFlashCommand(FLASH_COMMAND_STATUS);
      while (option!='q')
      {
    	  // read some data from the device
    	  // 129 is the endpoint address of the IN endpoint 4 (from PC to device)
    	  //dev.readInterrupt(129,readData,readData.length,5000,false);
    	  if (option !='h')
    	  {
    		 // int bytesread=LibusbJava.usb_interrupt_read(usb_device_handle,129,readData,readData.length,5000);
    		   int bytesread=LibusbJava.usb_bulk_read(usb_device_handle,129,readData,readData.length,5000);    	  
    		  if (bytesread>=0)
    			  Parse(readData,bytesread);
    		  if (ready_to_convertdata)
    		  {
    			  ready_to_convertdata=false;
    			  ConvertBinaryToCSVFile("sensordata.bin","data.csv");
    		  }    		  
    	  }
    	  if (flash_init)
    	  {
	    	  if (msp_connection_ready && !read_data_from_MSP)
	    	  {	    		 
	    		  System.out.print(">>");
	    		  if (preparing>0)
	    		  {
	    			  System.out.println("preparing");
	    			 //read the clock again
	    			  	if (preparing==1)
	    			  		option='r';
	    			  	else if (preparing==2)
	    			  		option='e';
	    			  	else if (preparing==3)
	    			  		option='s';
						preparing++;
						if (preparing>3)
						{
							preparing=0;
							System.out.println("Finish sensor preparation");
						}
						try {
							Thread.currentThread().sleep(500);
							
						}
						catch (InterruptedException ie)
						{
							
						}
	    		  }
	    		  else {
	    		  try {
	    			  InputStreamReader isr = new InputStreamReader(System.in);
	    			  BufferedReader br = new BufferedReader(isr);
	    			  String s = br.readLine();
	    			  byte [] bytearray=s.getBytes();
	    			  if (bytearray.length >0)
	    				  option =  bytearray[0];
	    		  }
	    		  catch (IOException e) 
	    		  { 
	    			  option=0;   	  
	    		  }
	    		  }	    		 
	    		  switch (option)
	    		  {
	    		  	case 'r'://read RTC clock
		    			  	SendMSPCommand(MSP_INTERFACE_READ_CLOCK_REQUEST);
		    			  	option=0;
		    			  	read_data_from_MSP=true;
		    			  	break;
	    		  	case 's'://set RTC clock
	    		  			SetRealTimeClock();
	    		  			option=0;
	    		  			read_data_from_MSP=true;
	    		  			break;
	    		  	case 'h'://help menu
	    		  			helpmenu();
	    		  			break;
	    		  	case 'd'://start download
	    		  			createFile("sensordata.bin");
	    		  			println("created file");
	    		  			RequestEEPROMAccess(EEPROM_ACCESS_DOWNLOAD_DATA);
	    		  			read_data_from_MSP=true;
	    		  			downloadingsize=0;
	    		  			downloading=true;
	    		  			lasteofsectionsize=0;
	    		  			nosections=0;
	    		  			option=0;
	    		  			break;
	    		  	case 'p'://prepare sensor 
	    		  			preparing=1;
	    		  			SetRealTimeClock();
	    		  			option=0;
	    		  			read_data_from_MSP=true;	    		  			
	    		  			break;
	    		  	case 'e'://erase all pages
	    		  			erasing=true;
	    		  			read_data_from_MSP=true;
	    		  			option=0;	    		  			
	    		  			curerasesection=0;
	    		  			println("Erasing..");
	    		  			nosections=FLASH_NO_SECTIONS;
	    		  			RequestEEPROMAccess(EEPROM_ACCESS_ERASE_ALL_MEMMORY);//erase
	    		  			break;
	    		  	case 'f'://find first available free memory address
	    		  			option =0;
	    		  			//SendFlashCommand(FLASH_COMMAND_FIND_AVAILABLE);
	    		  			read_data_from_MSP=true;
	    		  			RequestEEPROMAccess(EEPROM_ACCESS_GET_FIRST_AVAILABLE_ADDR);
	    		  			break;
	    		  }  			      			  	    		 
	    	  }
	    	  else if (!msp_connection_ready) {
	    		  try{
	    			  //do what you want to do before sleeping
	    			  Thread.currentThread().sleep(500);//sleep for 1000 ms
	    			  //do what you want to do after sleeptig
	    			}
	    			catch(InterruptedException ie){
	    			//If this thread was intrrupted by nother thread 
	    			}
	    		  SendMSPCommand(MSP_INTERFACE_CHECK_CONNECTION);
	    	  }
    	  }
      }
      try {
      if (fout!=null)
    	  fout.close();
      }
      catch (Exception e){//Catch exception if any
		  System.err.println("Error: " + e.getMessage());
	  }
      //dev.close(); // close the device
      if (LibusbJava.usb_release_interface(usb_device_handle,0)<0)//release the interface
    	  println("Error in releasing interface!!");
      LibusbJava.usb_close(usb_device_handle);// close the device
    /*} catch (USBException e) {
      // if an exception occures during connect or read/write an exception is thrown
      e.printStackTrace();
    }*/
  }
}
